<?php

namespace Intergo\UserCallbackClient\Services;

use Intergo\UserCallbackClient\CacheManagers\UserCallbackCountManager;
use Intergo\UserCallbackClient\CacheManagers\UserQueueManager;
use Intergo\UserCallbackClient\Enums\UserCallbackEnum;
use Intergo\UserCallbackClient\Exceptions\InvalidCallbackData;
use Intergo\UserCallbackClient\ValueObjects\UserCallbackData;
use App\Jobs\SendUserCallback;
use Illuminate\Support\Facades\Redis;
use Illuminate\Redis\Connections\Connection;

class UserCallbackService
{
    protected UserCallbackCountManager $callbackCountManager;
    public array $userQueues;

    /**
     * UserCallbackService constructor
     *
     * Instantiate the Service outside of loops to avoid
     * multiple unnecessary cache calls
     *
     * @param Redis $redis
     */
    public function __construct(Redis $redis)
    {
        $this->callbackCountManager = new UserCallbackCountManager($redis);
        $this->userQueues = (new UserQueueManager($redis))->hGetAll();
    }

    /**
     * @param UserCallbackData $data
     * @param string $traceId Log Trace ID
     * @throws InvalidCallbackData
     */
    public function send(UserCallbackData $data, string $traceId)
    {
        if (empty($data->callbackUrl) || empty($data->payload)) {
            throw new InvalidCallbackData();
        }

        $priority = 3; // Urgent priority
        $jobsCount = 0; // This means we use a custom queue, no throttle needed

        if (!empty($this->userQueues[$data->userId])) {
            /** User has custom queue */
            $queue = UserCallbackEnum::CALLBACKS_QUEUE . "-" . $this->userQueues[$data->userId];
        } else {
            $queue = UserCallbackEnum::CALLBACKS_QUEUE;
            $jobsCount = $this->callbackCountManager->increment($data->userId);
            if ($jobsCount > 1) {
                $priority = 2; //high priority
            }
            if ($jobsCount > 5000) {
                $priority = 1; //mid-priority
            }
            if ($jobsCount > 50000) {
                $priority = 0; //low priority
            }
        }

        dispatch(new SendUserCallback($data, $priority, $jobsCount, $traceId))->onQueue($queue);
    }
}